@@ -967,3 +967,127 @@ def repackage_data(data,quantities):
         print("Invalid quantity list provided for repackaging JETTO data, returning input data!")
 
     return odata
+
+
+def generate_entry_info(fieldname):
+    """ Metadata lookup table for a limited set of common ex-file fields. Can be expanded as necessary. """
+    lookup = {
+        'RA': (None, 'Minor radius, normalised', 'XVEC1', '1.0', 1.0),
+        'XRHO': (None, 'Normalised toroidal flux', 'XVEC1', '1.0', 1.0),
+        'PSI': (None, 'Normalised poloidal flux', 'XVEC1', '1.0', 1.0),
+        'SPSI': (None, 'Sqrt of normalised poloidal flux', 'XVEC1', '1.0', 1.0),
+        'R': ('m', 'Minor radius', 'XVEC1', '0.01', 0.01),
+        'RHO': ('m', 'JETTO rho coordinate', 'XVEC1', '0.01', 0.01),
+        'PR': ('Pa', 'Pressure (from equilibrium)', 'XVEC1', '1.0', 1.0),
+        'Q': (None, 'q (safety factor)', 'XVEC1', '1.0', 1.0),
+        'NE': ('m-3', 'Electron Density', 'XVEC1', '1000000.0', 1000000.0),
+        'TE': ('eV', 'Electron Temperature', 'XVEC1', '1.0', 1.0),
+        'TI': ('eV', 'Ion Temperature', 'XVEC1', '1.0', 1.0),
+        'ZEFF': (None, 'Z-effective', 'XVEC1', '1.0', 1.0),
+        'ANGF': ('s-1', 'Angular Frequency', 'XVEC1', '1.0', 1.0),
+        'NIMP': ('m-3', 'Impurity 1 Density', 'XVEC1', '1000000.0', 1000000.0),
+        'NIMP2': ('m-3', 'Impurity 2 Density', 'XVEC1', '1000000.0', 1000000.0),
+        'NIMP3': ('m-3', 'Impurity 3 Density', 'XVEC1', '1000000.0', 1000000.0),
+        'NIMP4': ('m-3', 'Impurity 4 Density', 'XVEC1', '1000000.0', 1000000.0),
+        'NIMP5': ('m-3', 'Impurity 5 Density', 'XVEC1', '1000000.0', 1000000.0),
+        'NIMP6': ('m-3', 'Impurity 6 Density', 'XVEC1', '1000000.0', 1000000.0),
+        'NIMP7': ('m-3', 'Impurity 7 Density', 'XVEC1', '1000000.0', 1000000.0),
+        'TRQI': ('N m-2', 'Intrinsic Torque', 'XVEC1', '0.1', 0.1),
+        'PRAD': ('W m-3', 'Radiation', 'XVEC1', '0.1', 0.1),
+        'QNBE': ('W m-3', 'Power Density Electrons', 'XVEC1', '0.1', 0.1),
+        'QNBI': ('W m-3', 'Power Density Ions', 'XVEC1', '0.1', 0.1),
+        'SB1': ('m-3 s-1', 'Particle Source 1', 'XVEC1', '1000000.0', 1000000.0),
+        'SB2': ('m-3 s-1', 'Particle Source 2', 'XVEC1', '1000000.0', 1000000.0),
+        'JZNB': ('A m-2', 'NB Driven Curr.Dens', 'XVEC1', '1.0E7', 10000000.0),
+        'NB': ('m-3', 'Fast Ion Density', 'XVEC1', '1000000.0', 1000000.0),
+        'WFNB': ('J m-3', 'Fast Ion Energy Density', 'XVEC1', '0.1', 0.1),
+        'TORQ': ('N m-2', 'Torque', 'XVEC1', '0.1', 0.1),
+        'QRFE': ('W m-3', 'Power Density Electrons', 'XVEC1', '0.1', 0.1),
+        'QRFI': ('W m-3', 'Power Density Ions', 'XVEC1', '0.1', 0.1),
+        'RF': ('m-3', 'Fast Ion Density', 'XVEC1', '1000000.0', 1000000.0),
+        'WFRF': ('J m-3', 'Fast Ion Energy Density', 'XVEC1', '0.1', 0.1),
+        'QECE': ('W m-3', 'Power Density Electrons', 'XVEC1', '0.1', 0.1),
+        'JZEC': ('A m-2', 'ECRH Driven Curr.Dens', 'XVEC1', '1.0E7', 10000000.0),
+        'QEBE': ('W m-3', 'Power Density Electrons', 'XVEC1', '0.1', 0.1),
+        'QEBI': ('W m-3', 'Power Density Ions', 'XVEC1', '0.1', 0.1),
+        'JZEB': ('A m-2', 'EBW Driven Curr.Dens', 'XVEC1', '1.0E7', 10000000.0),
+        'CUR': ('A', 'Plasma Current', 'TVEC1', '1.0', 1.0)
+    }
+    entry = None
+    if fieldname in lookup:
+        entry = {'UNITS': None, 'DESC': '', 'SCSTR': '', 'XBASE': '', 'UID': '', 'DDA': '', 'DTYPE': '', 'SEQ': '0', 'SCALE': 1.0, 'LABEL': '', 'FORM': 'float', 'SECNUM': 5}
+        unit, desc, xbase, scstr, sc = lookup[fieldname]
+        entry['UNITS'] = unit
+        entry['DESC'] = desc
+        entry['SCSTR'] = scstr
+        entry['XBASE'] = xbase
+        entry['SCALE'] = sc
+        entry['LABEL'] = fieldname
+    return entry
+
+
+def convert_jsp_to_exfile(outname, runfolder='./', filename='jetto.jsp', outdir='./'):
+    """ Helper function to convert JSP to ex-file for iterative runs. """
+    lookup = {
+        'XA': 'RA',
+        'XRHO': 'XRHO',
+        'XPSI': 'PSI',
+        'XPSQ': 'SPSI',
+        'R': 'R',
+        'RHO': 'RHO',
+        'PR': 'PR',
+        'Q': 'Q',
+        'NE': 'NE',
+        'TE': 'TE',
+        'TI': 'TI',
+        'ZEFF': 'ZEFF',
+        'ANGF': 'ANGF',
+        'NIM1': 'NIMP',
+        'NIM2': 'NIMP2',
+        'NIM3': 'NIMP3',
+        'NIM4': 'NIMP4',
+        'NIM5': 'NIMP5',
+        'NIM6': 'NIMP6',
+        'NIM7': 'NIMP7',
+        'TRQI': 'TRQI',
+        'QRAD': 'PRAD',
+        'QNBE': 'QNBE',
+        'QNBI': 'QNBI',
+        'SBD1': 'SB1',
+        'SBD2': 'SB2',
+        'JZNB': 'JZNB',
+        'DNBD': 'NB',
+        'WNBD': 'WFNB',
+        'TORQ': 'TORQ',
+        'QRFE': 'QRFE',
+        'QRFI': 'QRFI',
+        'DRFD': 'RF',
+        'WRFD': 'WFRF',
+        'QECE': 'QECE',
+        'JZEC': 'JZEC',
+        'QEBE': 'QEBE',
+        'QEBI': 'QEBI',
+        'JZEB': 'JZEB'
+    }
+    status = 1
+    idir = Path(runfolder)
+    if idir.is_dir() and (idir / filename).is_file():
+        iname = idir / filename
+        idata = read_binary_file(str(iname.resolve()))
+        if idata is not None:
+            odata = create_exfile_structure(idata['DDA NAME'], int(idata['SHOT']))
+            odata['XVEC1'] = copy.deepcopy(idata['XVEC1'])
+            odata['TVEC1'] = copy.deepcopy(idata['TIME'])
+            for var, exvar in lookup.items():
+                if var in idata and np.abs(np.sum(idata[var])) > 1.0e-10:
+                    info = generate_entry_info(exvar)
+                    info['DDA'] = 'JSP'
+                    info['DTYPE'] = var
+                    odata['INFO'][exvar] = info
+                    odata[exvar] = copy.deepcopy(idata[var])
+            odir = Path(outdir)
+            if not odir.is_dir():
+                odir.mkdir(parents=True)
+            oname = odir / outname
+            status = write_binary_exfile(odata, output_file=str(oname))
+    return status
